# ***************************************************************************
# Support functions for RRAWFLOW
#
# loadHydroVars() - loads the variables needed for a hydro model run from
#                   a .csv file.
# loadHydroData() - load the data needed for a hydro model run from a 
#                   .csv file.
# plotHydro()     - plots the results from a hydro model run.
# plotIRFyears()  - plots the IRF with years on the x-axis
# plotIRFdays()  - plots the IRF with days on the x-axis

loadHydroVars = function(filename)
{
    # NOTE: Strings must be delimited with double-quotes NOT single-quotes.

    vars_in <- read.csv(filename, 
                        header=TRUE, 
                        sep=",", 
                        colClasses=c("character","numeric","character",
									"character","character"))

    # Create a named list of the variables
    thevars <- list()
    thevars[vars_in[,1]] <- vars_in[,2]

    return(thevars)
}   # end function loadHydroVars()
# ---------------------------------------------------------------------------


loadHydroData = function(filename)
{
    # For the rrawflow.r script to work the csv file should have data with
    # header (column) names for:
    #   tempMid
    #   prec
    #   WetDry
    #
    # For modelType==2 you will also need:
    #   StreamFlw
    #
    # For plotting purposes you should also include:
    #   decYear

    thedata <- read.csv(filename, 
                        header=TRUE, 
                        sep=",", 
                        colClasses=c("integer","integer","integer",
                                     "numeric","numeric","numeric",
                                     "numeric","numeric","integer"))
    return(thedata)
}   # end function loadHydroData()
# ---------------------------------------------------------------------------


loadControl = function(filename)
{ # Load the IRF control points to which the cubic spline will be fit
    thedata <- read.csv(filename, 
                        header=TRUE, 
                        sep=",", 
                        colClasses=c("character","numeric","numeric",
									 "character","numeric","numeric"))                                   
}


loadIRF = function(filename)
{
    # For the rrawflow.r script to work the csv file should have data with
    # header (column) names for:
	#	TimeStep
    #   hDryNP
    #   hWetNP

    thedata <- read.csv(filename, 
                        header=TRUE, 
                        sep=",", 
                        colClasses=c("integer","numeric","numeric"))                                     

    return(thedata)
}   # end function loadHydroData()
# ---------------------------------------------------------------------------


plotHydro = function(basedata, modeldata)
{
    # Plots results from RRAWFLOW to a png file
    # The supplied dataset variable must be the variable that contains the
    # output from the rrawflow() function.

    # Parameters:
    #   basedata  - the base data that was used to drive the hydro model
    #   modeldata - the output data from the hydro model

    # Named arrays of information for plotting each variable
    yOutPrefs   <- list(name="Modeled response", color="darkgoldenrod", 
                        linewidth=0.5, linetype="solid")
    springPrefs <- list(name="Obs response", color="steelblue4", 
                        linewidth=0.75, linetype="solid")
    soilPrefs   <- list(name="Soil index, scaled", color="black", 
                        linewidth=0.2, linetype="solid", ylimits=c(7,-0.025))
    wetdryPrefs <- list(name="Wet/Dry", color="green", linewidth=1.5, linetype="dotted")

	# Plot to a png file.
	# A pdf will automatically be generated when running from a Windows 
	# command prompt if the file type is not specified.
	png(file="Rplots.png", width=20, height=5, units="in", res=300)
	
    # Legend setup
    legLTY <- c(yOutPrefs$linetype, springPrefs$linetype, 
                soilPrefs$linetype, wetdryPrefs$lintetype)
    legCOL <- c(yOutPrefs$color, springPrefs$color, soilPrefs$color, wetdryPrefs$color)
    legTXT <- c(yOutPrefs$name, springPrefs$name, soilPrefs$name, wetdryPrefs$name)
    legLWD <- c(yOutPrefs$linewidth, springPrefs$linewidth, 
                soilPrefs$linewidth, wetdryPrefs$linewidth)

    # Plot hydro model yOut
    plot(modeldata[[1]],modeldata[[2]],
		xlim=c(1975,2015),
		ylim=c(0,5),
		 type="l",
         yaxs="i",
         xlab = "Year",
         ylab = "",
         col=yOutPrefs$color,
         lwd=yOutPrefs$linewidth)

    # Plot observed response
    lines(basedata$decYear,basedata$response,
          type="l",
          col=springPrefs$color,
          lwd=springPrefs$linewidth)

    par(new=TRUE)   # This allows us to plot with a secondary axis on 
                    # top of the original plot

    # Plot hydro model s (soil index)
    plot(modeldata[[1]],modeldata[[5]],
         type="l",
         ylim=soilPrefs$ylimits,
         yaxs="i",
         xaxt="n",
         yaxt="n",
         xlab = "",
         ylab = "",
         col=soilPrefs$color,
         lwd=soilPrefs$linewidth)

    # Plot the wet-dry periods from the spreadsheet
    lines(basedata$decYear,basedata$WetDry,
          col=wetdryPrefs$color,
          lty=wetdryPrefs$linetype,
          lwd=wetdryPrefs$linewidth)


    # Add a legend
    legend("bottomleft",
           legend = legTXT,
           lty    = legLTY,
           col    = legCOL,
           pt.lwd = legLWD)	

	dev.off()		   

}   # end of function plotHydro()
# ---------------------------------------------------------------------------


plotIRFyears = function(modeldata)
{	# Plots the IRFs

    # Named arrays of information for plotting each variable
	hDryPrefs   <- list(name="hDry", color="darkgoldenrod", 
                        linewidth=0.5, linetype="solid")
    hWetPrefs   <- list(name="hWet", color="steelblue4", 
                        linewidth=0.5, linetype="solid")

	# Plot to a png file.
	# A pdf will automatically be generated when running from a Windows 
	# command prompt if the file type is not specified.
	png(file="IRF_years.png", width=20, height=15, units="in", res=300)
	
    # Legend setup
    legLTY <- c(hDryPrefs$linetype,  hWetPrefs$linetype) 
    legCOL <- c(hDryPrefs$color,     hWetPrefs$color)
    legTXT <- c(hDryPrefs$name,      hWetPrefs$name)
    legLWD <- c(hDryPrefs$linewidth, hWetPrefs$linewidth)

    # Plot hDry line
    plot(modeldata[[7]]/365.25,modeldata[[3]],
		xlim=c(0,5),  # or (0,5.5)
		ylim=c(0,.1),
		type="l",
        yaxs="i",
        xlab = "Year",
        ylab = "IRF magnitude",
        col=hDryPrefs$color,
        lwd=hDryPrefs$linewidth)
		
	# Plot hDry points
	points(cntrl_pts$pdTime/365.25,cntrl_pts$pDry,
		col=hDryPrefs$color)
		 
    # Plot hWet
    lines(modeldata[[7]]/365.25,modeldata[[4]],
		type="l",
		yaxs="i",
        xlab = "Year",
        ylab = "IRF magnitude",		
        col=hWetPrefs$color,
        lwd=hWetPrefs$linewidth)	

	# Plot hWet points
	points(cntrl_pts$pwTime/365.25,cntrl_pts$pWet,
		col=hWetPrefs$color)		

    # Add a legend
    legend("bottomleft",
           legend = legTXT,
           lty    = legLTY,
           col    = legCOL,
           pt.lwd = legLWD)	

	dev.off()		   
	}   # end of function plotIRFdays()

	plotIRFdays = function(modeldata)
{	# Plots the IRFs

    # Named arrays of information for plotting each variable
	hDryPrefs   <- list(name="hDry", color="darkgoldenrod", 
                        linewidth=0.5, linetype="solid")
    hWetPrefs   <- list(name="hWet", color="steelblue4", 
                        linewidth=0.5, linetype="solid")

	# Plot to a png file.
	# A pdf will automatically be generated when running from a Windows 
	# command prompt if the file type is not specified.
	png(file="IRF_days.png", width=20, height=5, units="in", res=300)
	
    # Legend setup
    legLTY <- c(hDryPrefs$linetype,  hWetPrefs$linetype) 
    legCOL <- c(hDryPrefs$color,     hWetPrefs$color)
    legTXT <- c(hDryPrefs$name,      hWetPrefs$name)
    legLWD <- c(hDryPrefs$linewidth, hWetPrefs$linewidth)

    # Plot hDry line
    plot(modeldata[[7]],modeldata[[3]],
		xlim=c(0,2000),
		ylim=c(0,.1),
		type="l",
        yaxs="i",
        xlab = "Day",
        ylab = "IRF magnitude",
        col=hDryPrefs$color,
        lwd=hDryPrefs$linewidth)
		
	# Plot hDry points
	points(cntrl_pts$pdTime,cntrl_pts$pDry,
		col=hDryPrefs$color)
		 
    # Plot hWet
    lines(modeldata[[7]],modeldata[[4]],
		type="l",
		yaxs="i",
        xlab = "Year",
        ylab = "IRF magnitude",		
        col=hWetPrefs$color,
        lwd=hWetPrefs$linewidth)	

	# Plot hWet points
	points(cntrl_pts$pwTime,cntrl_pts$pWet,
		col=hWetPrefs$color)		

    # Add a legend
    legend("bottomleft",
           legend = legTXT,
           lty    = legLTY,
           col    = legCOL,
           pt.lwd = legLWD)	

	dev.off()		   
}   # end of function plotIRFyears()
# ---------------------------------------------------------------------------