# RRAWFLOW: Rainfall-Response Aquifer and Watershed Flow Model
# Version 1.10 released 2014

rrawflow = function(vars, input, cntrl_pts)
{
    # INPUT DATASETS
		# rf_vars.csv - Model variables. The first 12 variables define the parametric IRFs (IRFtype=1) 
		# rf_input.csv - Time-series input datasets
		# control_pts.csv - IRF ordinates to which a curve will be fit (applies to IRFtype = 2 and 4)		
		# rf_irf.csv - User-defined nonparametric IRF

	# Read model settings
	T_m = vars$T_m
	T_s = vars$T_s
	modelType = vars$modelType
	IRFtype = vars$IRFtype
	inputType = vars$inputType
	TempOpt = vars$TempOpt

	# modelType describes the system output
    if (modelType==1) { # Water-level model
        HeadShft=vars$hshift  # Head shift adjusts for abitrary sea-level datum
    } else if (modelType==2 || modelType==3) {# Springflow/streamflow/solute-transport model
        HeadShft=0 # No head shift needed
    }

    #stime = proc.time()
    #print(proc.time() - stime)

    # Preallocate arrays
    ylen = length(input$Day)
    h1 = rep(0, ylen)
    h2 = h1 
    h3 = h1 
    h4 = h1
    h5  = h1 
    h = h1 
	  hDry = h1
	  hWet = h1
    hStrm = h1 
    p = h1 
    Peff = h1 
    yOut = h1 
    s = h1

    # Assign variables
    c1 = vars$c1
    alpha = vars$alpha
    s_init = 0.3
    sblfrct = vars$sblfrct
    f = vars$f

	# PARAMETRIC IRFs ====================================================================
	if (IRFtype==1) {  # Use parametric IRFs
		print("Parametric IRF using gamma functions")

		# Assign temporary variables (this saves time because of a lookup penalty)
		lamda1 = vars$lamda1
		eta1 = vars$eta1
		lamda2 = vars$lamda2
		eta2 = vars$eta2
		lamda3 = vars$lamda3
		eta3 = vars$eta3
		lamda4 = vars$lamda4
		eta4 = vars$eta4	

		tcntr = 0.541 # About half of a time step (when eta=1, G=1)
		
        x <- (1:ylen) - tcntr
        t <- (1:ylen) - tcntr

        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		# Calculate IRF for quick flow - dry period (h1)
        # Calculate the denominator G
        G = sum(x^(eta1-1) * exp(-x))

        area1 <- h1[1]
        h1 <- vars$epsilon1 * lamda1^eta1 * t^(eta1-1) * exp(-lamda1*t) / G
        area1 <- area1 + sum(h1) 
		    # print(paste("area1 =",area1))

        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		# Calculate IRF for slow flow - dry period (h2)
        # Calculate the denominator G
        G = sum(x^(eta2-1) * exp(-x))

        area2 <- h2[1]
        h2 <- vars$epsilon2 * lamda2^eta2 * t^(eta2-1) * exp(-lamda2*t) / G
        area2 <- area2 + sum(h2)
        # print(paste("area2 =",area2))

        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		# Calculate IRF for quick flow - wet period (h3)
        # Calculate the denominator G
        G = sum(x^(eta3-1) * exp(-x))

        area3 <- h3[1]
        h3 <- vars$epsilon3 * lamda3^eta3 * t^(eta3-1) * exp(-lamda3*t) / G
        area3 <- area3 + sum(h3) 
        # print(paste("area3 =",area3))

        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		# Calculate IRF for quick flow - wet period (h4)
        # Calculate the denominator G
        G = sum(x^(eta4-1) * exp(-x))

        area4 <- h4[1]
        h4 <- vars$epsilon4 * lamda4^eta4 * t^(eta4-1) * exp(-lamda4*t) / G
        area4 <- area4 + sum(h4)
	      # print(paste("area4 =",area4))	

		hDry = h1 + h2 # total wet perid IRF
		hWet = h3 + h4 # total dry period IRF
	}	# end if (IRFtype=1 parametric IRFs) 
	
	
	# NONPARAMETRIC IRFs ===============================================================
	if (IRFtype==2)  { # Fit a cubic spline to the points
		# The first pTime value 1; all time values must be integers
		print("Nonparametric IRF using cubic spline fit to control points")	
		
		# Dry period
		# Set end of spline fit to the time of the last cntrl_pts point
		plen=length(cntrl_pts$pdTime)
		hEnd=cntrl_pts$pdTime[plen]
		
		hDry=spline(cntrl_pts$pdTime,cntrl_pts$pDry,n=hEnd,method="fmm",xmin=1,xmax=hEnd,ties="ordered")

		# Set the x (time) and y values for hDry after the end of the spline fit
		hDry$x[hEnd:ylen]=c(hEnd:ylen) 
		hDry$y[hEnd:ylen]=0   # Set all y values to zero after the end of the spline fit
				
		# Set any negative values from the spline fit to zero
		for (ii in 1:hEnd) {
			if (hDry$y[ii] < 0) {
				hDry$y[ii] = 0
			} # end if
		} # end for
		hDry=hDry$y

		# Wet period
		# Set end of spline fit to the time of the last cntrl_pts point
		plen=length(cntrl_pts$pwTime)
		hEnd=cntrl_pts$pwTime[plen]
		
		hWet=spline(cntrl_pts$pwTime,cntrl_pts$pWet,n=hEnd,method="fmm",xmin=1,xmax=hEnd,ties="ordered")
		# Set the x (time) and y values for hWet after the end of the spline fit
		hWet$x[hEnd:ylen]=c(hEnd:ylen) 
		hWet$y[hEnd:ylen]=0            # Set all y values to zero after the end of the spline fit
		
		# Set any negative values from the spline fit to zero
		for (ii in 1:hEnd) {
			if (hWet$y[ii]<0) {
				hWet$y[ii]=0
			} # end if
		} # end for
		hWet=hWet$y
		
	} else if (IRFtype==3) { # Fit straight lines between the points
		# The first pTime value 1; ; all time values must be integers
		print("Nonparametric IRF with a straight line between control points")
		
		# Dry period
		nstps = length(cntrl_pts$pdTime)-1 # number of intervals between points
		for (i in 1:nstps) {
			# Set linefit value at cntrl_pts point
			j = cntrl_pts$pdTime[i]
			hDry[j] = cntrl_pts$pDry[i]
			# calculate y increment
			yinc = (cntrl_pts$pDry[i+1]-cntrl_pts$pDry[i])/(cntrl_pts$pdTime[i+1]-cntrl_pts$pdTime[i]) 
			# Add incremental change between cntrl_pts points
			for (j in (cntrl_pts$pdTime[i]+1):(cntrl_pts$pdTime[i+1])) {
				hDry[j] = hDry[j-1]+yinc
			}
		}

		# Wet period
		nstps = length(cntrl_pts$pwTime)-1 # number of intervals between points
		for (i in 1:nstps) {
			# Set linefit value at cntrl_pts point
			j = cntrl_pts$pwTime[i]
			hWet[j] = cntrl_pts$pWet[i]

			# calculate y increment
			yinc = (cntrl_pts$pWet[i+1]-cntrl_pts$pWet[i])/(cntrl_pts$pwTime[i+1]-cntrl_pts$pwTime[i]) 

			# Add incremental change between cntrl_pts points
			for (j in (cntrl_pts$pwTime[i]+1):(cntrl_pts$pwTime[i+1])) {
				hWet[j] = hWet[j-1]+yinc
			}
		}
	} else if (IRFtype==4) { # Nonparametric IRF pre-defined
		print("Nonparametric IRF read from rf_irf.csv")
		hDry=irf$hDryNP
		hWet=irf$hWetNP		
	} # end if	(IRFtype= 2, 3, or 4 nonparametric IRFs




	# Set the input variable p (precipitation, sinking-stream rate, or solute concentration)
	# This is the "input" column in the input file
    p = input$input 
	
	if (inputType==1) { # System input is precipitation

        # Calculate p for winter using air temperature (snow precip)
        wntr = 0
        for (ii in 1:ylen) {
            if (input$tempMean[ii] < T_s) {
                wntr = p[ii] + wntr        # calculate sum of snow precip
                p[ii] = 0                  # snow precip is held until a warming period
            }
            if (input$tempMean[ii] > T_m) {
                p[ii] = (1-sblfrct) * wntr + p[ii]  # add snowmelt to this day
                wntr = 0
            }
        }

        # Calculate effective precip or intiltration recharge (Peff)
        # Kpa = Kappa
        Kpa = rep(0,ylen)
        if (TempOpt==2) {   # Neglect air temperature (Kpa is a scalar)
            Kpa = alpha
            #for (ii in 1:ylen) {
            #    Kpa[ii]=alpha
            #}
        } else if (TempOpt==1) {    # Adjust Kpa based on air temperature (Kpa is a vector)
            Kpa = alpha * exp((20 - input$tempMean) * f)
            #for (ii in 1:ylen) {
            #    Kpa[ii] = alpha * exp((20 - input$tempMean[ii]) * f)
            #}
        }

        s[1] = s_init
        Peff[1] = p[1] * s[1] # Initial value of Peff
        b4 = 1 - 1/Kpa 

        for (ii in 2:ylen) {
            s[ii] = c1 * p[ii] + b4[ii] * s[ii-1]

            if (s[ii] > 1) {
                s[ii] = 1 # Peff cannot exceed precipitation
            } else if (s[ii] < 0) {
                s[ii] = 0
            }
        }
        Peff = p * s    # Peff is effective precipitation or infiltration recharge

	} else if (inputType==2 || inputType==3) { # System input is a sinking stream or solute concentration
		Peff = p # System input should be entered in "input" column in the input file
	}

    # Split Peff into wet and dry periods
    PeffD = rep(0, ylen)
    PeffW = rep(0, ylen)
    PeffW <- ifelse(input$WetDry == 1, Peff, 0)
    PeffD <- ifelse(input$WetDry != 1, Peff, 0)

	# Time-variance mode
	# This determines how to do the convolution
	if (vars$timeVarMode==1) {# Time invariant (static IRF)
		# Use static dry-period IRF (ignore wet-period IRF)
		yOut = convolve(Peff, rev(hDry), type='open')
	} else if (vars$timeVarMode==2) {# use wet & dry period IRFs
		yOutD = convolve(PeffD, rev(hDry), type='open') # Dry-period response
		yOutW = convolve(PeffW, rev(hWet), type='open') # Wet-period response
		yOut = yOutD + yOutW # Superposition of wet and dry period responses
	} else if (vars$timeVarMode==3) {# IRF vertical scale is variable
		# beta varies with moving (MA) average of Peff
		# Use dry-period IRF (ignore wet-period IRF)
		# Caluclate MA of the input to convolution (Peff)
        stime = proc.time()
		shift=round((vars$MAwindow-1)/2,0)  # 1/2 of MA window
		MA = rep(0,ylen)    # initialize
		beta1 = rep(0,ylen) # initialize
		
        for (i in (shift+1):(ylen-shift)) { # MA loop
			MA[i]=mean(Peff[(i-shift):(i+shift)])# calculate mean of window
		}

		# Set start and end of record to first and last MA value (constant)
		MA[1:shift] = MA[shift+1] # start of record
		MA[(ylen-shift+1):ylen] = MA[ylen-shift] # end of record
		MA = (MA / max(MA))   # scale MA to range from 0 to 1
        print("Compute MA timing")
        print(proc.time() - stime)

        beta1 <- vars$slope * MA

        stime = proc.time()
        tmp = beta1 * Peff
		# Convolution loop with variable beta
        for (i in 1:ylen) {
            hDryREV = rev(hDry[1:i]) # rev reverses the vector
            terms = tmp[1:i] * hDryREV[1:i] # convolution
            yOut[i] = yOut[i] + sum(terms)
        }
		yOut = yOut[1:ylen] # Truncate yOut to length of input
        print("Convolution Timing")
        print(proc.time() - stime)
	}
	
    # Shorten array to length of simualtion period 
    # Add HeadShft to match measured values at arbitrary datum
    yOut = yOut[1:ylen] + HeadShft 
	
	# Write output to rf_output.csv
	tDay=c(1:ylen)
    return(list("decYear"=input$decYear,"response"=yOut,"hDry"=hDry,
                "hWet"=hWet,"s"=s,"conv_input"=Peff,"tDay"=tDay))
					
	# Assess model fit
	ssr=sum(input$response-yOut)^2 # sum of the squared residuals
	ssdm=sum(input$response-mean(input$response))^2  # sum of the squared deviations from the mean for observed response
	E=1-ssr/ssdm # Nash?Sutcliffe coefficient of efficiency
				
}   # end function rrawflow
